<?php

namespace Drupal\coupons_migrate\Commands;

use Drupal\commerce_promotion\Entity\Coupon;
use Drupal\commerce_promotion\Entity\Promotion;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Database;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drush\Commands\DrushCommands;

/**
 * Drush command class for migrating coupons.
 */
class CouponsMigrate extends DrushCommands {

  /**
   * The logger channel factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new CouponsMigrate object.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger channel factory.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(LoggerChannelFactoryInterface $loggerFactory, Connection $database, EntityTypeManagerInterface $entityTypeManager) {
    $this->loggerFactory = $loggerFactory;
    $this->database = $database;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Migrates coupons from a legacy database.
   *
   * @param string $key
   *   Argument bundle (legacy-db-key).
   *
   * @command migrate-coupons:migrateCoupons
   * @aliases migrate-coupons
   */
  public function migrateCoupons($key) {
    $connection = Database::getConnection('default', $key);
    Database::setActiveConnection($key);

    $query = $connection->select('uc_coupons', 'uc')
      ->fields('uc', ['cid', 'code', 'type', 'value', 'valid_until',
        'data', 'name', 'created', 'max_uses', 'status',
      ])
      ->condition('uc.status', 1);
    $coupons = $query->execute()->fetchAll();

    // Counter for migrated coupons.
    $migratedCount = 0;

    foreach ($coupons as $coupon) {
      $code = $coupon->code;
      $existingCoupon = $this->checkCoupon($code);

      if (empty($existingCoupon)) {
        $offer = $this->getOffer($coupon->type, $coupon->value);
        $endDate = $this->getEndDate($coupon->valid_until);
        $skus = $this->getSkus(unserialize($coupon->data, ['allowed_classes' => FALSE]));
        $promotion = Promotion::create([
          'name' => $coupon->name,
          'order_types' => ['default'],
          'offer' => $offer,
          'start_date' => date('Y-m-d H:i:s', $coupon->created),
          'end_date' => $endDate,
          'usage_limit' => $coupon->max_uses,
          'status' => TRUE,
        ]);
        $promotion->save();

        if (!empty($skus)) {
          $conditions = $this->createConditions($skus);
          if ($conditions !== NULL) {
            $promotion->set('conditions', $conditions);
            $promotion->save();
          }
        }
        $newCoupon = Coupon::create([
          'promotion_id' => $promotion->id(),
          'code' => $coupon->code,
          'usage_limit' => $coupon->max_uses,
          'status' => TRUE,
        ]);
        $newCoupon->save();

        if ($newCoupon->save()) {
          $this->loggerFactory->get('coupons_migrate')->notice("Coupon {$code} has been migrated.");
          $migratedCount++;
        }
      }
    }
    $this->loggerFactory->get('coupons_migrate')->notice("Total migrated coupons: {$migratedCount}");
  }

  /**
   * Checks if a coupon already exists by code.
   *
   * @param string $code
   *   The coupon code.
   *
   * @return string|null
   *   The coupon code if found, NULL otherwise.
   */
  private function checkCoupon($code) {
    $connection = $this->database;
    $query = $connection->select('commerce_promotion_coupon', 'cpc')
      ->fields('cpc', ['code'])
      ->condition('cpc.code', $code);

    return $query->execute()->fetchField();
  }

  /**
   * Generates the offer configuration based on type and value.
   *
   * @param string $type
   *   The type of the coupon.
   * @param float $value
   *   The value of the coupon.
   *
   * @return array
   *   The offer configuration.
   */
  private function getOffer($type, $value) {
    switch ($type) {
      case 'price':
        return [
          'target_plugin_id' => 'order_fixed_amount_off',
          'target_plugin_configuration' => [
            'amount' => [
              'number' => $value,
              'currency_code' => 'USD',
            ],
          ],
        ];

      case 'percentage':
        return [
          'target_plugin_id' => 'order_percentage_off',
          'target_plugin_configuration' => [
            'percentage' => $value / 100,
          ],
        ];

      case 'set_price':
        return [
          'target_plugin_id' => 'set_price',
          'target_plugin_configuration' => [
            'amount' => [
              'number' => $value,
              'currency_code' => 'USD',
            ],
          ],
        ];

      default:
        return [];
    }
  }

  /**
   * Converts valid_until timestamp to the required format.
   *
   * @param int|null $validUntil
   *   The valid_until timestamp.
   *
   * @return string|null
   *   The formatted date or NULL if not provided.
   */
  private function getEndDate($validUntil) {
    return $validUntil ? date('Y-m-d H:i:s', $validUntil) : NULL;
  }

  /**
   * Extracts SKUs from the coupon data.
   *
   * @param array $data
   *   The unserialized coupon data.
   *
   * @return array
   *   The list of SKUs.
   */
  private function getSkus(array $data): array {
    return $data['skus'] ?? [];
  }

  /**
   * Creates promotion conditions for the provided SKUs.
   *
   * @param array $skus
   *   The list of SKUs.
   *
   * @return array
   *   The conditions configuration.
   */
  private function createConditions(array $skus) {
    $entities = [];
    $query = $this->database->select('commerce_product_variation_field_data', 'cpv')
      ->fields('cpv', ['sku'])
      ->condition('cpv.sku', $skus, 'IN');
    $results = $query->execute()->fetchAll();
    if (!empty($results)) {
      foreach ($results as $sku) {
        $product_variation = $this->loadVariationBySku($sku->sku);
        if ($product_variation) {
          $entities[] = $product_variation->uuid();
        }
      }

      return [[
        'target_plugin_id' => 'order_purchased_entity:commerce_product_variation',
        'target_plugin_configuration' => ['entities' => $entities],
      ],
      ];
    }
    else {
      return NULL;
    }
  }

  /**
   * Loads a product variation by SKU.
   *
   * @param string $sku
   *   The product SKU.
   *
   * @return \Drupal\commerce_product\Entity\ProductVariation|null
   *   The product variation or NULL if not found.
   */
  private function loadVariationBySku($sku) {
    $variations = $this->entityTypeManager
      ->getStorage('commerce_product_variation')
      ->loadByProperties(['sku' => $sku]);
    return !empty($variations) ? reset($variations) : NULL;
  }

}
